#ifndef GRAPH_BELLMAN_FORD_H_
#define GRAPH_BELLMAN_FORD_H_

#include <core/multipleDispatch.hpp>
#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph {

namespace shortestpath {

// Bellman-Ford only works on graphs without negative-weight cycle
// However it will detect the negative-weight cycle and throw exception
//
// note: negative-weight undirected edge = negative-weight cycle

class BellmanFord : public std::SingleDispatchLastStaticParam<BellmanFord, std::unordered_map<Node, int>, graph::GraphBase, const Node&> {
public:
	typedef std::unordered_map<Node, int> Result;

	static Result bellmanford(const Graph &graph, const Node &start);

	static Result bellmanford(const DirectedGraph &graph, const Node &start);
	static Result bellmanford(const UndirectedGraph &graph, const Node &start);
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_BELLMAN_FORD_H_
