/*
 * CommonException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "CommonException.h"

#include "../sax/FromXMLParserHelper.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"
#include "../global/GlobalData.h"

#ifdef DEBUG
	#include "../debug/simpleStacktrace.h"
#endif

namespace exception {

CommonException::CommonException ( std::string cause ) : cause(std::move(cause)) {
	#ifdef DEBUG
		this->backtrace = std::simpleStacktrace();
	#else
		this->backtrace = "";
	#endif

	for(int i = 0; i < common::GlobalData::argc; i++) {
		if(i != 0) this->command += " ";
		this->command += common::GlobalData::argv[i];
	}
	this->whatMessage += this->backtrace;
	this->whatMessage += this->cause;
}

CommonException::CommonException ( std::string cause, std::string backtrace, std::string command ) : cause(std::move(cause)), backtrace(std::move(backtrace)), command(std::move(command)) {
	this->whatMessage += this->backtrace;
	this->whatMessage += this->cause;
}

alib::ObjectBase* CommonException::clone() const {
	return new CommonException(*this);
}

alib::ObjectBase* CommonException::plunder() && {
	return new CommonException(std::move(*this));
}

int CommonException::compare(const CommonException& other) const {
	return this->whatMessage.compare(other.whatMessage);
}

const char * CommonException::what ( ) const noexcept {
	return whatMessage.c_str ( );
}

const std::string & CommonException::getCause ( ) const {
	return cause;
}

const std::string & CommonException::getBacktrace ( ) const {
	return backtrace;
}

void CommonException::operator>>(std::ostream& os) const {
	os << this->whatMessage;
}

CommonException::operator std::string() const {
	return this->whatMessage;
}

const std::string CommonException::XML_TAG_NAME = "CommonException";

CommonException CommonException::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, CommonException::XML_TAG_NAME);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "cause");
	std::string cause = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		cause = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "cause");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "backtrace");
	std::string backtrace = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		backtrace = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "backtrace");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "command");
	std::string command = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		command = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "command");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, CommonException::XML_TAG_NAME);
	return CommonException(std::move(cause), std::move(backtrace), std::move(command));
}

void CommonException::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(CommonException::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("cause", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(cause, sax::Token::TokenType::CHARACTER);
	out.emplace_back("cause", sax::Token::TokenType::END_ELEMENT);
	out.emplace_back("backtrace", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(backtrace, sax::Token::TokenType::CHARACTER);
	out.emplace_back("backtrace", sax::Token::TokenType::END_ELEMENT);
	out.emplace_back("command", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(command, sax::Token::TokenType::CHARACTER);
	out.emplace_back("command", sax::Token::TokenType::END_ELEMENT);
	out.emplace_back(CommonException::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

} /* namespace exception */

namespace alib {

auto alibExceptionParserRegister2 = xmlApi<alib::Object>::ParserRegister<exception::CommonException>();

} /* namespace alib */
