/*
 * LabeledSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#ifndef LABELED_SYMBOL_H_
#define LABELED_SYMBOL_H_

#include "../label/Label.h"
#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class LabeledSymbol : public SymbolBase {
protected:
	label::Label label;

public:
	explicit LabeledSymbol ( int number );
	explicit LabeledSymbol ( char character );
	explicit LabeledSymbol ( std::string label );

	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabeledSymbol ( label::Label label );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const label::Label & getLabel ( ) const;

	label::Label & getLabel ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const LabeledSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static LabeledSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* LABELED_SYMBOL_H_ */
