/*
 * NonlinearVariableSymbol.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "NonlinearVariableSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

NonlinearVariableSymbol::NonlinearVariableSymbol ( int number ) : symbol ( alphabet::symbolFrom ( number ) ) {
}

NonlinearVariableSymbol::NonlinearVariableSymbol ( char character ) : symbol ( alphabet::symbolFrom ( character ) ) {
}

NonlinearVariableSymbol::NonlinearVariableSymbol ( std::string symbol ) : symbol ( alphabet::symbolFrom ( std::move ( symbol ) ) ) {
}

NonlinearVariableSymbol::NonlinearVariableSymbol ( alphabet::Symbol symbol ) : symbol ( std::move ( symbol ) ) {
}

SymbolBase * NonlinearVariableSymbol::clone ( ) const {
	return new NonlinearVariableSymbol ( * this );
}

SymbolBase * NonlinearVariableSymbol::plunder ( ) && {
	return new NonlinearVariableSymbol ( std::move ( * this ) );
}

int NonlinearVariableSymbol::compare ( const NonlinearVariableSymbol & other ) const {
	return this->symbol.getData ( ).compare ( other.symbol.getData ( ) );
}

void NonlinearVariableSymbol::operator >>( std::ostream & out ) const {
	out << "(NonlinearVariableSymbol " << symbol << ")";
}

NonlinearVariableSymbol::operator std::string ( ) const {
	return "$" + ( std::string ) symbol;
}

const std::string NonlinearVariableSymbol::XML_TAG_NAME = "NonlinearVariableSymbol";

NonlinearVariableSymbol NonlinearVariableSymbol::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, NonlinearVariableSymbol::XML_TAG_NAME );
	alphabet::Symbol data = alib::xmlApi < alphabet::Symbol >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, NonlinearVariableSymbol::XML_TAG_NAME );

	return NonlinearVariableSymbol ( data );
}

void NonlinearVariableSymbol::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( NonlinearVariableSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( NonlinearVariableSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

SymbolBase * NonlinearVariableSymbol::next ( ) const {
	return new UniqueSymbol ( Symbol ( * this ), primitive::Integer ( 0 ) );
}

SymbolBase * NonlinearVariableSymbol::inc ( ) && {
	return new UniqueSymbol ( Symbol ( std::move ( * this ) ), primitive::Integer ( 0 ) );
}

} /* namespace alphabet */

namespace alib {

auto nonlinearVariableSymbolParserRegister	= xmlApi < alphabet::Symbol >::ParserRegister < alphabet::NonlinearVariableSymbol > ( );
auto nonlinearVariableSymbolParserRegister2 = xmlApi < alib::Object >::ParserRegister < alphabet::NonlinearVariableSymbol > ( );

} /* namespace alib */
