/*
 * NonlinearVariableSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef NONLINEAR_VARIABLE_SYMBOL_H_
#define NONLINEAR_VARIABLE_SYMBOL_H_

#include "Symbol.h"

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
class NonlinearVariableSymbol : public SymbolBase {
	alphabet::Symbol symbol;

public:
	explicit NonlinearVariableSymbol ( int number );
	explicit NonlinearVariableSymbol ( char character );
	explicit NonlinearVariableSymbol ( std::string symbol );
	explicit NonlinearVariableSymbol ( alphabet::Symbol symbol );

	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit NonlinearVariableSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const NonlinearVariableSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static NonlinearVariableSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* NONLINEAR_VARIABLE_SYMBOL_H_ */
