/*
 * SubtreeWildcardSymbol.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "SubtreeWildcardSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

SubtreeWildcardSymbol::SubtreeWildcardSymbol() {

}

SymbolBase* SubtreeWildcardSymbol::clone() const {
	return new SubtreeWildcardSymbol(*this);
}

SymbolBase* SubtreeWildcardSymbol::plunder() && {
	return new SubtreeWildcardSymbol(std::move(*this));
}

int SubtreeWildcardSymbol::compare(const SubtreeWildcardSymbol&) const {
	return 0;
}

void SubtreeWildcardSymbol::operator>>(std::ostream& out) const {
	out << "(SubtreeWildcardSymbol)";
}

SubtreeWildcardSymbol::operator std::string () const {
	return "#S";
}

SubtreeWildcardSymbol SubtreeWildcardSymbol::SUBTREE_WILDCARD = SubtreeWildcardSymbol ( );
Symbol SubtreeWildcardSymbol::SUBTREE_WILDCARD_SYMBOL = Symbol ( SubtreeWildcardSymbol ( ) );
RankedSymbol SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL = RankedSymbol ( Symbol ( SubtreeWildcardSymbol ( ) ), 0);

const std::string SubtreeWildcardSymbol::XML_TAG_NAME = "SubtreeWildcardSymbol";

SubtreeWildcardSymbol SubtreeWildcardSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, SubtreeWildcardSymbol::XML_TAG_NAME);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, SubtreeWildcardSymbol::XML_TAG_NAME);
	return SubtreeWildcardSymbol();
}

void SubtreeWildcardSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(SubtreeWildcardSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(SubtreeWildcardSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* SubtreeWildcardSymbol::next() const {
	return new UniqueSymbol(Symbol(*this), primitive::Integer(0));
}

SymbolBase* SubtreeWildcardSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

auto subtreeWildcardSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::SubtreeWildcardSymbol>();
auto subtreeWildcardSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::SubtreeWildcardSymbol>();

} /* namespace alib */
