/*
 * SymbolPairSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_PAIR_SYMBOL_H_
#define SYMBOL_PAIR_SYMBOL_H_

#include "Symbol.h"
#include <pair>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class SymbolPairSymbol : public SymbolBase {
protected:
	std::pair < Symbol, Symbol > symbol;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit SymbolPairSymbol ( std::pair < Symbol, Symbol > symbol );

	virtual SymbolBase * clone ( ) const;

	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair < Symbol, Symbol > & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SymbolPairSymbol & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static SymbolPairSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* SYMBOL_PAIR_SYMBOL_H_ */
