/*
 * UniqueSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_ID_SYMBOL_H_
#define SYMBOL_ID_SYMBOL_H_

#include "Symbol.h"
#include "../primitive/Integer.h"

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class UniqueSymbol : public SymbolBase {
protected:
	Symbol symbol;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueSymbol ( Symbol symbol, primitive::Integer id );

	virtual SymbolBase * clone ( ) const;

	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const Symbol & getSymbol ( ) const;

	const primitive::Integer & getId ( ) const;

	primitive::Integer & getId ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UniqueSymbol & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static UniqueSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* SYMBOL_ID_SYMBOL_H_ */
