/*
 * CallReturnLocalInputAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "CallReturnLocalInputAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getCallInputAlphabet() const {
	return callInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addCallInputSymbol(alphabet::Symbol symbol) {
	if(returnInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in return alphabet");
	if(localInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in local alphabet");
	return callInputAlphabet.insert(std::move(symbol)).second;
}

void CallReturnLocalInputAlphabet::setCallInputAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(callInputAlphabet.begin(), callInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	callInputAlphabet = std::move(newSymbols);
}

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getReturnInputAlphabet() const {
	return returnInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addReturnInputSymbol(alphabet::Symbol symbol) {
	if(localInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in local alphabet");
	if(callInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in call alphabet");
	return returnInputAlphabet.insert(std::move(symbol)).second;
}

void CallReturnLocalInputAlphabet::setReturnInputAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(returnInputAlphabet.begin(), returnInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	returnInputAlphabet = std::move(newSymbols);
}

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getLocalInputAlphabet() const {
	return localInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addLocalInputSymbol(alphabet::Symbol symbol) {
	if(callInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in call alphabet");
	if(returnInputAlphabet.count(symbol))
		throw AutomatonException("Input symbol " + (std::string) symbol + " already in return alphabet");
	return localInputAlphabet.insert(std::move(symbol)).second;
}

void CallReturnLocalInputAlphabet::setLocalInputAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(localInputAlphabet.begin(), localInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	localInputAlphabet = std::move(newSymbols);
}

} /* namespace automaton */

