/*
 * List.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_LIST_H_
#define OBJECTS_LIST_H_

#include <list>
#include <string>

#include <object/Object.h>
#include "ContainerBase.h"

#include <XmlApi.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsList : public std::list < alib::Object >, public ContainerBase {
public:
	explicit ObjectsList ( std::list < alib::Object > );
	explicit ObjectsList ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsList & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static ObjectsList parse ( std::deque < sax::Token >::iterator & input );
	template < class T >
	static std::list < T > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	template < class T >
	static void compose ( std::deque < sax::Token > & out, const std::list < T > & input );
};

template < class T >
std::list < T > ObjectsList::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsList::XML_TAG_NAME );

	std::list < T > list;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		list.push_back ( alib::xmlApi < T >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsList::XML_TAG_NAME );
	return list;
}

template < class T >
void ObjectsList::compose ( std::deque < sax::Token > & out, const std::list < T > & input ) {
	out.emplace_back ( ObjectsList::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );

	for ( const T & item : input )
		alib::xmlApi < T >::compose ( out, item );

	out.emplace_back ( ObjectsList::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < typename T >
struct xmlApi < std::list < T > > {
	static std::list < T > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const std::list < T > & data );
};

template < typename T >
std::list < T > xmlApi < std::list < T > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsList::parseRaw < T > ( input );
}

template < typename T >
bool xmlApi < std::list < T > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, container::ObjectsList::XML_TAG_NAME );
}

template < typename T >
void xmlApi < std::list < T > >::compose ( std::deque < sax::Token > & output, const std::list < T > & input ) {
	return container::ObjectsList::compose < T > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_LIST_H_ */
