/*
 * Map.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_MAP_H_
#define OBJECTS_MAP_H_

#include <map>
#include <string>

#include <object/Object.h>
#include "ContainerBase.h"
#include "ObjectsPair.h"

#include <XmlApi.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsMap : public std::map < alib::Object, alib::Object >, public ContainerBase {
public:
	explicit ObjectsMap ( std::map < alib::Object, alib::Object > );
	explicit ObjectsMap ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsMap & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static ObjectsMap parse ( std::deque < sax::Token >::iterator & input );
	template < class T, class R >
	static std::map < T, R > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	template < class T, class R >
	static void compose ( std::deque < sax::Token > & out, const std::map < T, R > & input );
};

template < class T, class R >
std::map < T, R > ObjectsMap::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsMap::XML_TAG_NAME );

	std::map < T, R > map;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		map.insert ( alib::xmlApi < std::pair < T, R > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsMap::XML_TAG_NAME );
	return map;
}

template < class T, class R >
void ObjectsMap::compose ( std::deque < sax::Token > & out, const std::map < T, R > & input ) {
	out.emplace_back ( ObjectsMap::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );

	for ( const std::pair < const T, R > & item : input )
		alib::xmlApi < std::pair < T, R > >::compose ( out, item );

	out.emplace_back ( ObjectsMap::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < typename T, typename R >
struct xmlApi < std::map < T, R > > {
	static std::map < T, R > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const std::map < T, R > & data );
};

template < typename T, typename R >
std::map < T, R > xmlApi < std::map < T, R > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsMap::parseRaw < T, R > ( input );
}

template < typename T, typename R >
bool xmlApi < std::map < T, R > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, container::ObjectsMap::XML_TAG_NAME );
}

template < typename T, typename R >
void xmlApi < std::map < T, R > >::compose ( std::deque < sax::Token > & output, const std::map < T, R > & input ) {
	return container::ObjectsMap::compose < T, R > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_MAP_H_ */
