/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_SET_H_
#define OBJECTS_SET_H_

#include <set>
#include <string>

#include <object/Object.h>
#include "ContainerBase.h"

#include <XmlApi.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsSet : public std::set < alib::Object >, public ContainerBase {
public:
	explicit ObjectsSet ( std::set < alib::Object > );
	explicit ObjectsSet ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsSet & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static ObjectsSet parse ( std::deque < sax::Token >::iterator & input );
	template < class T >
	static std::set < T > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	template < class T >
	static void compose ( std::deque < sax::Token > & out, const std::set < T > & input );
};

template < class T >
std::set < T > ObjectsSet::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsSet::XML_TAG_NAME );

	std::set < T > set;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		set.insert ( alib::xmlApi < T >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsSet::XML_TAG_NAME );
	return set;
}

template < class T >
void ObjectsSet::compose ( std::deque < sax::Token > & out, const std::set < T > & input ) {
	out.emplace_back ( ObjectsSet::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );

	for ( const T & item : input )
		alib::xmlApi < T >::compose ( out, item );

	out.emplace_back ( ObjectsSet::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < typename T >
struct xmlApi < std::set < T > > {
	static std::set < T > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const std::set < T > & data );
};

template < typename T >
std::set < T > xmlApi < std::set < T > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsSet::parseRaw < T > ( input );
}

template < typename T >
bool xmlApi < std::set < T > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, container::ObjectsSet::XML_TAG_NAME );
}

template < typename T >
void xmlApi < std::set < T > >::compose ( std::deque < sax::Token > & output, const std::set < T > & input ) {
	return container::ObjectsSet::compose < T > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_SET_H_ */
