/*
 * vector.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_VECTOR_H_
#define OBJECTS_VECTOR_H_

#include <vector>
#include <string>

#include <object/Object.h>
#include "ContainerBase.h"

#include <XmlApi.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsVector : public std::vector < alib::Object >, public ContainerBase {
public:
	explicit ObjectsVector ( std::vector < alib::Object > );
	explicit ObjectsVector ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsVector & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static ObjectsVector parse ( std::deque < sax::Token >::iterator & input );
	template < class T >
	static std::vector < T > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	template < class T >
	static void compose ( std::deque < sax::Token > & out, const std::vector < T > & input );
};

template < class T >
std::vector < T > ObjectsVector::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsVector::XML_TAG_NAME );

	std::vector < T > vector;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		vector.push_back ( alib::xmlApi < T >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsVector::XML_TAG_NAME );
	return vector;
}

template < class T >
void ObjectsVector::compose ( std::deque < sax::Token > & out, const std::vector < T > & input ) {
	out.emplace_back ( ObjectsVector::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );

	for ( const T & item : input )
		alib::xmlApi < T >::compose ( out, item );

	out.emplace_back ( ObjectsVector::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < typename T >
struct xmlApi < std::vector < T > > {
	static std::vector < T > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const std::vector < T > & data );
};

template < typename T >
std::vector < T > xmlApi < std::vector < T > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsVector::parseRaw < T > ( input );
}

template < typename T >
bool xmlApi < std::vector < T > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, container::ObjectsVector::XML_TAG_NAME );
}

template < typename T >
void xmlApi < std::vector < T > >::compose ( std::deque < sax::Token > & output, const std::vector < T > & input ) {
	return container::ObjectsVector::compose < T > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_VECTOR_H_ */
