/*
 * SuffixTrieFinalMark.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SuffixTrieFinalMark.h"
#include "SuffixTrieTerminatingSymbol.h"
#include <exception/CommonException.h>

#include <iostream>
#include <algorithm>
#include <sstream>

#include <sax/FromXMLParserHelper.h>
#include "../common/IndexFromXMLParser.h"
#include "../common/IndexToXMLComposer.h"
#include <object/Object.h>
#include <XmlApi.hpp>

namespace indexes {

SuffixTrieFinalMark::SuffixTrieFinalMark ( std::set < alphabet::Symbol > alphabet ) : SuffixTrieFinalMark ( std::move ( alphabet ), SuffixTrieNodeFinalMark ( { }, true ) ) {
}

SuffixTrieFinalMark::SuffixTrieFinalMark ( std::set < alphabet::Symbol > alphabet, SuffixTrieNodeFinalMark tree ) : std::Components < SuffixTrieFinalMark, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabet ) ), std::tuple < > ( ) ), tree ( NULL ) {
	setTree ( std::move ( tree ) );
}

SuffixTrieFinalMark::SuffixTrieFinalMark ( SuffixTrieNodeFinalMark tree ) : SuffixTrieFinalMark ( tree.computeMinimalAlphabet ( ), tree ) {
}

SuffixTrieFinalMark::SuffixTrieFinalMark ( const SuffixTrieTerminatingSymbol & other ) : std::Components < SuffixTrieFinalMark, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( other.getAlphabet ( ) ), std::tuple < > ( ) ), tree ( NULL ) {
	this->accessComponent < GeneralAlphabet > ( ).remove ( other.accessElement < TerminatingSymbol > ( ).get ( ) );
	setTree ( SuffixTrieNodeFinalMark ( other.getRoot ( ), other.accessElement < TerminatingSymbol > ( ).get ( ) ) );
}

SuffixTrieFinalMark::SuffixTrieFinalMark ( const SuffixTrieFinalMark & other ) : std::Components < SuffixTrieFinalMark, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( other.getAlphabet ( ) ), std::tuple < > ( ) ), tree ( other.tree->clone ( ) ) {
	this->tree->attachTree ( this );
}

SuffixTrieFinalMark::SuffixTrieFinalMark ( SuffixTrieFinalMark && other ) noexcept : std::Components < SuffixTrieFinalMark, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( other.accessComponent < GeneralAlphabet > ( ).get ( ) ) ), std::tuple < > ( ) ), tree ( other.tree ) {
	this->tree->attachTree ( this );
	other.tree = NULL;
}

alib::ObjectBase * SuffixTrieFinalMark::clone ( ) const {
	return new SuffixTrieFinalMark ( * this );
}

alib::ObjectBase * SuffixTrieFinalMark::plunder ( ) && {
	return new SuffixTrieFinalMark ( std::move ( * this ) );
}

SuffixTrieFinalMark & SuffixTrieFinalMark::operator =( const SuffixTrieFinalMark & other ) {
	if ( this == & other )
		return * this;

	* this = SuffixTrieFinalMark ( other );

	return * this;
}

SuffixTrieFinalMark & SuffixTrieFinalMark::operator =( SuffixTrieFinalMark && other ) noexcept {
	std::swap ( this->tree, other.tree );
	std::swap ( accessComponent < GeneralAlphabet > ( ).get ( ), other.accessComponent < GeneralAlphabet > ( ).get ( ) );

	return * this;
}

SuffixTrieFinalMark::~SuffixTrieFinalMark ( ) noexcept {
	delete tree;
}

const SuffixTrieNodeFinalMark & SuffixTrieFinalMark::getRoot ( ) const {
	return * tree;
}

SuffixTrieNodeFinalMark & SuffixTrieFinalMark::getRoot ( ) {
	return * tree;
}

void SuffixTrieFinalMark::setTree ( SuffixTrieNodeFinalMark tree ) {
	delete this->tree;
	this->tree = std::move ( tree ).plunder ( );

	if ( !this->tree->attachTree ( this ) ) {
		delete this->tree;
		throw exception::CommonException ( "Input symbols not in the alphabet." );
	}
}

void SuffixTrieFinalMark::operator >>( std::ostream & out ) const {
	out << "(SuffixTrieFinalMark " << * ( this->tree ) << ")";
}

std::ostream & operator <<( std::ostream & out, const SuffixTrieFinalMark & instance ) {
	instance >> out;
	return out;
}

int SuffixTrieFinalMark::compare ( const SuffixTrieFinalMark & other ) const {
	auto first = std::tie ( * tree, getAlphabet() );
	auto second = std::tie ( * other.tree, other.getAlphabet() );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

SuffixTrieFinalMark::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

const std::string SuffixTrieFinalMark::XML_TAG_NAME = "SuffixTrieFinalMark";

SuffixTrieFinalMark SuffixTrieFinalMark::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, SuffixTrieFinalMark::XML_TAG_NAME );
	std::set < alphabet::Symbol > rankedAlphabet = IndexFromXMLParser::parseAlphabet ( input );
	SuffixTrieNodeFinalMark * root = IndexFromXMLParser::parseSuffixTrieNodeFinalMark ( input );
	SuffixTrieFinalMark tree ( std::move ( rankedAlphabet ), std::move ( * root ) );

	delete root;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, SuffixTrieFinalMark::XML_TAG_NAME );
	return tree;
}

void SuffixTrieFinalMark::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( SuffixTrieFinalMark::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );
	IndexToXMLComposer::composeAlphabet ( out, getAlphabet ( ) );
	IndexToXMLComposer::composeNode ( out, * tree );
	out.emplace_back ( SuffixTrieFinalMark::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace indexes */

namespace std {

template < >
bool indexes::SuffixTrieFinalMark::Component < indexes::SuffixTrieFinalMark, alphabet::Symbol, indexes::GeneralAlphabet >::used ( const alphabet::Symbol & symbol ) const {
	return static_cast < const indexes::SuffixTrieFinalMark * > ( this )->getRoot ( ).testSymbol ( symbol );
}

template < >
bool indexes::SuffixTrieFinalMark::Component < indexes::SuffixTrieFinalMark, alphabet::Symbol, indexes::GeneralAlphabet >::available ( const alphabet::Symbol & ) const {
	return true;
}

template < >
void indexes::SuffixTrieFinalMark::Component < indexes::SuffixTrieFinalMark, alphabet::Symbol, indexes::GeneralAlphabet >::valid ( const alphabet::Symbol & ) const {
}

} /* namespace std */

namespace alib {

auto suffixTreeFinalMarkParserRegister = xmlApi < alib::Object >::ParserRegister < indexes::SuffixTrieFinalMark > ( );

} /* namespace alib */
