/*
 * SuffixTrieFinalMark.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUFFIX_TRIE_FINAL_MARK_H_
#define SUFFIX_TRIE_FINAL_MARK_H_

#include <vector>
#include <list>
#include <string>
#include <set>
#include <core/components.hpp>
#include "SuffixTrieNodeFinalMark.h"
#include <object/ObjectBase.h>

namespace indexes {

class SuffixTrieTerminatingSymbol;
class GeneralAlphabet;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of RegExpElement.
 */
class SuffixTrieFinalMark : public alib::ObjectBase, public std::Components < SuffixTrieFinalMark, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > {
protected:
	SuffixTrieNodeFinalMark * tree;

public:
	/**
	 * @copydoc SuffixTrieNodeFinalMark::clone() const
	 */
	virtual ObjectBase * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::plunder() const
	 */
	virtual ObjectBase * plunder ( ) &&;

	explicit SuffixTrieFinalMark ( std::set < alphabet::Symbol > alphabet );
	explicit SuffixTrieFinalMark ( std::set < alphabet::Symbol > alphabet, SuffixTrieNodeFinalMark tree );
	explicit SuffixTrieFinalMark ( SuffixTrieNodeFinalMark tree );

	explicit SuffixTrieFinalMark ( const SuffixTrieTerminatingSymbol & tree );

	/**
	 * Copy constructor.
	 * @param other tree to copy
	 */
	SuffixTrieFinalMark ( const SuffixTrieFinalMark & other );
	SuffixTrieFinalMark ( SuffixTrieFinalMark && other ) noexcept;
	SuffixTrieFinalMark & operator =( const SuffixTrieFinalMark & other );
	SuffixTrieFinalMark & operator =( SuffixTrieFinalMark && other ) noexcept;
	~SuffixTrieFinalMark ( ) noexcept;

	/**
	 * @return Root node of the regular expression tree
	 */
	const SuffixTrieNodeFinalMark & getRoot ( ) const;

	/**
	 * @return Root node of the regular expression tree
	 */
	SuffixTrieNodeFinalMark & getRoot ( );

	const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	/**
	 * Sets the root node of the regular expression tree
	 * @param tree root node to set
	 */
	void setTree ( SuffixTrieNodeFinalMark tree );

	/**
	 * Removes symbol from the alphabet of symbol available in the regular expression
	 * @param symbol removed symbol from the alphabet
	 */
	bool removeSymbolFromAlphabet ( const alphabet::Symbol & symbol );

	/**
	 * Prints XML representation of the tree to the output stream.
	 * @param out output stream to which print the tree
	 * @param tree tree to print
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream & out, const SuffixTrieFinalMark & instance );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SuffixTrieFinalMark & other ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static SuffixTrieFinalMark parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

#endif /* SUFFIX_TRIE_FINAL_MARK_H_ */
