/*
 * SuffixTrieTerminatingSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUFFIX_TRIE_TERMINATING_SYMBOL_H_
#define SUFFIX_TRIE_TERMINATING_SYMBOL_H_

#include <vector>
#include <list>
#include <string>
#include <set>
#include <core/components.hpp>
#include "SuffixTrieNodeTerminatingSymbol.h"
#include <object/ObjectBase.h>

namespace indexes {

class GeneralAlphabet;
class TerminatingSymbol;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of RegExpElement.
 */
class SuffixTrieTerminatingSymbol : public alib::ObjectBase, public std::Components < SuffixTrieTerminatingSymbol, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < TerminatingSymbol > > {
protected:
	SuffixTrieNodeTerminatingSymbol * tree;

public:
	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::clone() const
	 */
	virtual ObjectBase * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::plunder() const
	 */
	virtual ObjectBase * plunder ( ) &&;

	explicit SuffixTrieTerminatingSymbol ( std::set < alphabet::Symbol > alphabet, alphabet::Symbol terminatingSymbol );
	explicit SuffixTrieTerminatingSymbol ( std::set < alphabet::Symbol > alphabet, alphabet::Symbol terminatingSymbol, SuffixTrieNodeTerminatingSymbol tree );
	explicit SuffixTrieTerminatingSymbol ( alphabet::Symbol terminatingSymbol, SuffixTrieNodeTerminatingSymbol tree );

	/**
	 * Copy constructor.
	 * @param other tree to copy
	 */
	SuffixTrieTerminatingSymbol ( const SuffixTrieTerminatingSymbol & other );
	SuffixTrieTerminatingSymbol ( SuffixTrieTerminatingSymbol && other ) noexcept;
	SuffixTrieTerminatingSymbol & operator =( const SuffixTrieTerminatingSymbol & other );
	SuffixTrieTerminatingSymbol & operator =( SuffixTrieTerminatingSymbol && other ) noexcept;
	~SuffixTrieTerminatingSymbol ( ) noexcept;

	/**
	 * @return Root node of the regular expression tree
	 */
	const SuffixTrieNodeTerminatingSymbol & getRoot ( ) const;

	/**
	 * @return Root node of the regular expression tree
	 */
	SuffixTrieNodeTerminatingSymbol & getRoot ( );

	/**
	 * Sets the root node of the regular expression tree
	 * @param tree root node to set
	 */
	void setTree ( SuffixTrieNodeTerminatingSymbol tree );

	const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	const alphabet::Symbol & getTerminatingSymbol ( ) const {
		return accessElement < TerminatingSymbol > ( ).get ( );
	}

	/**
	 * Prints XML representation of the tree to the output stream.
	 * @param out output stream to which print the tree
	 * @param tree tree to print
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream & out, const SuffixTrieTerminatingSymbol & instance );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SuffixTrieTerminatingSymbol & other ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static SuffixTrieTerminatingSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

#endif /* SUFFIX_TRIE_TERMINATING_SYMBOL_H_ */
