/*
 * LabelPairLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_PAIR_LABEL_H_
#define LABEL_PAIR_LABEL_H_

#include "Label.h"
#include <pair>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelPairLabel : public LabelBase {
protected:
	std::pair < Label, Label > label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelPairLabel ( std::pair < Label, Label > label );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair < Label, Label > & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const LabelPairLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static LabelPairLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * next ( ) const;
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

#endif /* LABEL_PAIR_LABEL_H_ */
