/*
 * UniqueLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_ID_LABEL_H_
#define LABEL_ID_LABEL_H_

#include "Label.h"
#include "../primitive/Integer.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class UniqueLabel : public LabelBase {
protected:
	Label label;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueLabel ( Label label, primitive::Integer id );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const Label & getLabel ( ) const;

	const primitive::Integer & getId ( ) const;

	primitive::Integer & getId ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UniqueLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static UniqueLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * next ( ) const;
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

#endif /* LABEL_ID_LABEL_H_ */
