/*
 * Bool.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Bool.h"
#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include "Primitive.h"
#include <object/Object.h>

namespace primitive {

Bool::Bool(bool data) : data(data) {

}

PrimitiveBase* Bool::clone() const {
	return new Bool(*this);
}

PrimitiveBase* Bool::plunder() && {
	return new Bool(std::move(*this));
}

bool Bool::getData() const {
	return data;
}

void Bool::setData(bool data) {
	this->data = data;
}

int Bool::compare(const Bool& other) const {
	return data - other.data;
}

void Bool::operator>>(std::ostream& out) const {
	out << "(Bool " << (data ? "true" : "false") << ")";
}

Bool::operator std::string() const {
	return data ? "true" : "false";
}

const std::string Bool::XML_TAG_NAME = "Bool";

Bool Bool::parse(std::deque<sax::Token>::iterator& input) {
	return Bool(parseRaw(input));
}

bool Bool::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Bool::XML_TAG_NAME);
	std::string tmp = sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER);
	bool data;
	if(tmp == "true")
		data = true;
	else if(tmp == "false")
		data = false;
	else
		throw exception::CommonException("Invalid boolean value");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Bool::XML_TAG_NAME);
	return data;
}

void Bool::compose(std::deque<sax::Token>& out) const {
	compose(out, getData());
}

void Bool::compose(std::deque<sax::Token>& out, bool primitive) {
	out.emplace_back(Bool::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	if(primitive)
		out.emplace_back("true", sax::Token::TokenType::CHARACTER);
	else
		out.emplace_back("false", sax::Token::TokenType::CHARACTER);
	out.emplace_back(Bool::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

Bool Bool::next() const {
	return Bool(!data);
}

void Bool::inc() {
	data = !data;
}

} /* namespace primitive */

namespace alib {

bool xmlApi<bool>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Bool::parseRaw(input);
}

bool xmlApi<bool>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::Bool::XML_TAG_NAME);
}

void xmlApi<bool>::compose(std::deque<sax::Token>& output, bool data) {
	primitive::Bool::compose(output, data);
}

auto boolParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::Bool>();
auto boolParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::Bool>();

} /* namespace alib */
