/*
 * Epsilon.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "Epsilon.h"

#include <sax/FromXMLParserHelper.h>
#include "common/StringFromXMLParser.h"
#include "common/StringToXMLComposer.h"
#include "String.h"
#include <object/Object.h>
#include <XmlApi.hpp>

namespace string {

Epsilon::Epsilon(std::set<alphabet::Symbol> alphabet) : std::Components < Epsilon, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabet ) ), std::tuple < > ( ) ) {
}

Epsilon::Epsilon() : Epsilon ( std::set< alphabet::Symbol > ( ) ) {
}

StringBase* Epsilon::clone() const {
	return new Epsilon(*this);
}

StringBase* Epsilon::plunder() && {
	return new Epsilon(std::move(*this));
}

const std::vector<alphabet::Symbol> & Epsilon::getContent() const {
	return Epsilon::content;
}

bool Epsilon::isEmpty() const {
	return true;
}

int Epsilon::compare(const Epsilon& other) const {
	std::compare<std::set<alphabet::Symbol>> comp;
	return comp(getAlphabet(), other.getAlphabet());
}

void Epsilon::operator>>(std::ostream& out) const {
	out << "(Epsilon)";
}

Epsilon::operator std::string() const {
	return "E";
}

Epsilon Epsilon::EPSILON = Epsilon();

const std::string Epsilon::XML_TAG_NAME = "Epsilon";

std::vector<alphabet::Symbol> Epsilon::content;

Epsilon Epsilon::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Epsilon::XML_TAG_NAME);
	std::set<alphabet::Symbol> alphabet = StringFromXMLParser::parseAlphabet(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Epsilon::XML_TAG_NAME);
	return Epsilon(alphabet);
}

void Epsilon::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(Epsilon::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	StringToXMLComposer::compose(out, getAlphabet());
	out.emplace_back(Epsilon::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

} /* namespace string */

namespace std {

template < >
bool ::string::Epsilon::Component < ::string::Epsilon, alphabet::Symbol, ::string::GeneralAlphabet >::used ( const alphabet::Symbol & ) const {
	return false;
}

template < >
bool ::string::Epsilon::Component < ::string::Epsilon, alphabet::Symbol, ::string::GeneralAlphabet >::available ( const alphabet::Symbol & ) const {
	return true;
}

template < >
void ::string::Epsilon::Component < ::string::Epsilon, alphabet::Symbol, ::string::GeneralAlphabet >::valid ( const alphabet::Symbol & ) const {
}

} /* namespace std */

namespace alib {

auto epsilonParserRegister = xmlApi<string::String>::ParserRegister<string::Epsilon>();
auto epsilonParserRegister2 = xmlApi<alib::Object>::ParserRegister<string::Epsilon>();

} /* namespace alib */
