/*
 * algorithm.hpp
 *
 * Created on: May 28, 2015
 * Author: Jan Travnicek
 */

#ifndef ALGORITHM_HPP_
#define ALGORITHM_HPP_

namespace std {

template<class InputIt1, class InputIt2, class Compare>
bool excludes(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2, Compare comp) {
	while (first2 != last2 && first1 != last1) {
		if (comp(*first2, *first1)) {
			++first2;
		} else if (comp(*first1, *first2)) {
			++first1;
		} else {
			return false;
		}
	}
	return true;
}

template<class InputIt1, class InputIt2>
bool excludes(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2) {
	return excludes(first1, last1, first2, last2, std::less<decltype(*first1)>());
}

template<class ResType, class InType, template < typename ... > class ContainerType, class Callback >
ContainerType<ResType> transform(const ContainerType<InType> & in, Callback transform) {
	ContainerType<ResType> res;
	std::transform ( in.begin ( ), in.end ( ), std::inserter ( res, res.begin ( ) ), transform );
	return res;
}

} /* namespace std */

#endif /* ALGORITHM_HPP_ */
